@@ -7,7 +7,7 @@ import torch.nn.functional
 from typing import List, Tuple
 import math
 
-none = None
+none: None = None
 
 
 def check_type(x, type_v):
