@@ -345,7 +345,7 @@ pub fn expand_maybe_diag(
 
     // if no diag, just expand
     if is_unique(in_ints) && is_unique(out_ints) {
-        return Rearrange::nrc(
+        return Rearrange::nrc_elim_identity(
             node.clone(),
             RearrangeSpec::expand(in_ints, out_ints, int_sizes),
             None,
@@ -368,7 +368,7 @@ pub fn expand_maybe_diag(
             .chain(ints_to_add_before_einsum.iter())
             .cloned()
             .collect();
-        result = Rearrange::nrc(
+        result = Rearrange::nrc_elim_identity(
             result,
             RearrangeSpec::expand(
                 &(0_u8..in_ints.len() as u8).collect(),
@@ -394,7 +394,7 @@ pub fn expand_maybe_diag(
 
     // add non-diag broadcast dims
     if cur_ints[..] != out_ints[..] {
-        result = Rearrange::nrc(
+        result = Rearrange::nrc_elim_identity(
             result,
             RearrangeSpec::expand(
                 &(0_u8..out_ints.len() as u8)
@@ -434,7 +434,7 @@ pub fn make_broadcast(
         .collect();
     let output_ints: RInts = (0..(out_shape.len())).map(|i| sv![i as u8]).collect();
     let int_sizes = out_shape.clone();
-    Ok(Rearrange::nrc(
+    Ok(Rearrange::nrc_elim_identity(
         node.clone(),
         RearrangeSpec::new_canon(input_ints, output_ints, shape_to_op_shape(&int_sizes)),
         None,
@@ -573,11 +573,11 @@ pub fn remove_axes(circuit: &CircuitRc, axes: &HashSet<usize>) -> Option<Circuit
         Circuit::Rearrange(rearrange) => {
             // canonicalize first to make input 1s (), allowing us to remove them from output
             let spec = rearrange.spec.canonicalize(true);
-            Some(
-                rearrange
-                    .evolve(None, Some(spec.filter_out_axes_unsafe(axes)))
-                    .rc(),
-            )
+            Some(Rearrange::nrc_elim_identity(
+                rearrange.node.clone(),
+                spec.filter_out_axes_unsafe(axes),
+                rearrange.name_cloned(),
+            ))
         }
         _ => {
             if circuit
@@ -587,7 +587,7 @@ pub fn remove_axes(circuit: &CircuitRc, axes: &HashSet<usize>) -> Option<Circuit
                 .enumerate()
                 .all(|(i, l)| !axes.contains(&i) || *l == 1)
             {
-                Some(Rearrange::nrc(
+                Some(Rearrange::nrc_elim_identity(
                     circuit.clone(),
                     // wow this chained construction is long, should change at some point
                     RearrangeSpec::ident(circuit.info().rank())
@@ -628,7 +628,7 @@ pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<Cir
         .iter()
         .map(|x| x + node.info().rank() - node.nodes[0].info().rank())
         .collect();
-    Some(Rearrange::nrc(
+    Some(Rearrange::nrc_elim_identity(
         new_generalfunction.rc(),
         RearrangeSpec::unremove_axes(&removable_batchable_out, &node.info().shape),
         None,
@@ -662,7 +662,7 @@ pub fn concat_pull_removable_axes(node: &Concat) -> Option<CircuitRc> {
         node.name_cloned(),
     )
     .unwrap();
-    Some(Rearrange::nrc(
+    Some(Rearrange::nrc_elim_identity(
         new_node.rc(),
         RearrangeSpec::unremove_axes(&removable_non_axis, &node.info().shape),
         None,
@@ -827,12 +827,16 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
                     output_ints,
                     shape_to_op_shape(&raw_removed.info().shape),
                 );
-                Rearrange::nrc(raw_removed, broadcast_rspec, None)
+                Rearrange::nrc_elim_identity(raw_removed, broadcast_rspec, None)
             }
         })
         .collect();
     let add_circuit = Add::nrc(new_operands, add.name_cloned());
-    Some(Rearrange::nrc(add_circuit, post_rearrange_spec, None))
+    Some(Rearrange::nrc_elim_identity(
+        add_circuit,
+        post_rearrange_spec,
+        None,
+    ))
 }
 
 #[pyfunction]
@@ -1294,7 +1298,7 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
             });
             python_result.map(|(index, spec)| {
                 let new_index = Index::nrc(inner.node.clone(), index, node.name_cloned());
-                Rearrange::nrc(new_index, spec, inner.name_cloned())
+                Rearrange::nrc_elim_identity(new_index, spec, inner.name_cloned())
             })
         }
         Circuit::Scatter(inner) => {
@@ -1364,7 +1368,7 @@ pub fn concat_repeat_to_rearrange(concat: &Concat) -> Option<Concat> {
                 did_anything = true;
                 let mut int_sizes = prev_node.info().shape.clone();
                 int_sizes.push(same_seen);
-                new_nodes.push(Rearrange::nrc(
+                new_nodes.push(Rearrange::nrc_elim_identity(
                     prev_node.clone(),
                     RearrangeSpec {
                         input_ints: (0..prev_node.info().rank()).map(|x| sv![x as u8]).collect(),
