@@ -340,7 +340,7 @@ impl Einsum {
     }
 
     #[staticmethod]
-    #[pyo3(name = "from_einsum_string")]
+    #[pyo3(name = "None")]
     pub fn from_einsum_string(
         string: String,
         nodes: Vec<CircuitRc>,
@@ -363,7 +363,7 @@ impl Einsum {
     }
 
     #[staticmethod]
-    #[pyo3(name = "from_spec")]
+    #[pyo3(name = "None")]
     pub fn from_spec_py(
         spec: EinsumSpec,
         circuits: Vec<CircuitRc>,
