@@ -1,5 +1,7 @@
 use crate::hashmaps::FxHashMap as HashMap;
+use crate::rrfs::{arrayconstant_from_hash, save_tensor_rrfs};
 use crate::{new_rc, pyo3_prelude::*, sv};
+use base16::encode_lower;
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
 use pyo3::types::{IntoPyDict, PyTuple};
@@ -111,7 +113,7 @@ impl ArrayConstant {
 impl ArrayConstant {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(name = "\"ArrayConstant\".to_owned()")]
+    #[args(name = "None")]
     fn py_new(value: Tensor, name: Option<String>) -> PyClassInitializer<Self> {
         ArrayConstant::new(value, name).into_init()
     }
@@ -177,6 +179,19 @@ impl ArrayConstant {
         });
         Self::randn_named(shape, name, device_dtype)
     }
+
+    pub fn save_rrfs(&self) -> Result<String, PyErr> {
+        save_tensor_rrfs(self.value.clone())
+    }
+
+    pub fn tensor_hash_base16(&self) -> String {
+        encode_lower(&self.value.hash().unwrap())
+    }
+
+    #[staticmethod]
+    pub fn from_hash(name: Option<String>, hash_base16: &str) -> Result<Self, PyErr> {
+        arrayconstant_from_hash(name, hash_base16)
+    }
 }
 
 #[pyclass(extends=PyCircuitBase, unsendable)]
@@ -243,7 +258,7 @@ impl Symbol {
 impl Symbol {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(name = "\"Symbol\".to_owned()")]
+    #[args(name = "None")]
     fn py_new(shape: Shape, uuid: PyUuid, name: Option<String>) -> PyClassInitializer<Self> {
         Symbol::new(shape, uuid.0, name).into_init()
     }
@@ -317,7 +332,7 @@ impl ScalarConstant {
 impl ScalarConstant {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(shape = "sv![]", name = "\"ScalarConstant\".to_owned()")]
+    #[args(shape = "sv![]", name = "None")]
     fn py_new(value: f64, shape: Shape, name: Option<String>) -> PyClassInitializer<Self> {
         Self::new(value, shape, name).into_init()
     }
