@@ -323,7 +323,7 @@ impl CircuitNodeAutoName for GeneralFunction {
 impl GeneralFunction {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(name = "\"GeneralFunction\".to_owned()")]
+    #[args(name = "None")]
     fn new(
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
