@@ -1282,6 +1282,11 @@ impl PyCircuitBase {
         PyHashBytes(self.info().hash)
     }
 
+    #[getter]
+    fn hash_base16(&self) -> String {
+        base16::encode_lower(&self.info().hash)
+    }
+
     fn __hash__(&self) -> u64 {
         u64::from_le_bytes(
             self.info().hash[..std::mem::size_of::<u64>()]
