@@ -3,7 +3,8 @@ use uuid::Uuid;
 
 use super::{OpSize, RearrangeSpec, TensorAxisIndex, TensorIndex};
 use crate::{
-    all_imports::TorchDeviceDtypeOp, opt_einsum::EinsumSpec, smallvec::Sv, tensor_util::Shape,
+    all_imports::TorchDeviceDtypeOp, opt_einsum::EinsumSpec, rrfs::save_tensor_rrfs, smallvec::Sv,
+    tensor_util::Shape,
 };
 
 pub trait RustRepr {
@@ -123,17 +124,30 @@ impl RustRepr for ReprWrapper {
 }
 
 impl TensorAxisIndex {
-    pub fn repr(&self, bound: usize, device_dtype: &TorchDeviceDtypeOp) -> String {
+    pub fn repr(
+        &self,
+        bound: usize,
+        device_dtype: &TorchDeviceDtypeOp,
+        tensors_as_randn: bool,
+    ) -> String {
         match self {
             TensorAxisIndex::Single(i) => format!("TensorAxisIndex::Single({})", i),
             TensorAxisIndex::Tensor(t) => {
-                format!(
-                    "TensorAxisIndex::new_tensor_randint_seeded({}, {}, {}, {})",
-                    t.shape()[0].repr(),
-                    bound.repr(),
-                    device_dtype.repr(),
-                    t.hash_usize().unwrap(),
-                )
+                if tensors_as_randn {
+                    format!(
+                        "TensorAxisIndex::new_tensor_randint_seeded({}, {}, {}, {})",
+                        t.shape()[0].repr(),
+                        bound.repr(),
+                        device_dtype.repr(),
+                        t.hash_usize().unwrap(),
+                    )
+                } else {
+                    let hash_base16: &str = &save_tensor_rrfs(t.clone()).unwrap();
+                    format!(
+                        "TensorAxisIndex::Tensor(tensor_from_hash({}).unwrap())",
+                        hash_base16.repr()
+                    )
+                }
             }
             TensorAxisIndex::Slice(s) => format!(
                 "TensorAxisIndex::Slice(Slice {{ start:{}, stop:{} }})",
@@ -145,14 +159,23 @@ impl TensorAxisIndex {
 }
 
 impl TensorIndex {
-    pub fn repr(&self, shape: Shape, device_dtype: &TorchDeviceDtypeOp) -> String {
+    pub fn repr(
+        &self,
+        shape: Shape,
+        device_dtype: &TorchDeviceDtypeOp,
+        tensors_as_randn: bool,
+    ) -> String {
         // shape: shape of the tensor we're indexing into
         format!(
             "TensorIndex ( vec![{}] )",
             self.0
                 .iter()
                 .enumerate()
-                .map(|(i, tensor_axis_index)| tensor_axis_index.repr(shape[i], device_dtype))
+                .map(|(i, tensor_axis_index)| tensor_axis_index.repr(
+                    shape[i],
+                    device_dtype,
+                    tensors_as_randn
+                ))
                 .collect::<Vec<String>>()
                 .join(", ")
         )
