@@ -182,6 +182,8 @@ pub enum SchedulingError {
         node_memories: Vec<usize>,
         string: String,
     },
+    #[error("Single element doesn't fit {numel}")]
+    SingleElementTooBig { numel: BigUint },
 }
 
 create_exception!(
@@ -640,12 +642,20 @@ pub fn circuits_to_dag(circuits: &[&Circuit]) -> Dag {
     result
 }
 
-fn simplify_dag_for_scheduling_and_kept_nodes(dag: &Dag, chunk_size: usize) -> (Dag, Vec<usize>) {
+fn simplify_dag_for_scheduling_and_kept_nodes(
+    dag: &Dag,
+    chunk_size: usize,
+    lossy: bool,
+) -> (Dag, Vec<usize>) {
     // filter out nodes that are too small to be worth scheduling (such as cost 0 indexes)
-    let nodes_to_keep: Vec<usize> = (0..dag.node_costs.len())
-        .filter(|i| dag.node_costs[*i] >= chunk_size)
-        .collect();
-    let dag = dag.sub_dag(&nodes_to_keep);
+    let mut dag = dag.clone();
+    if lossy {
+        let nodes_to_keep: Vec<usize> = (0..dag.node_costs.len())
+            .filter(|i| dag.node_costs[*i] >= chunk_size)
+            .collect();
+        dag = dag.sub_dag(&nodes_to_keep);
+    }
+
     // now we filter out some parts in chains of serial computation
     // if we have a chain of nodes `start -> node_1 -> ... -> end`,
     // we can simplify the dag to only include the node with the smallest cost,
@@ -730,39 +740,56 @@ pub fn circuit_to_schedule(
     circuit: CircuitRc,
     context: &mut OptimizationContext,
 ) -> Result<Schedule, SchedulingError> {
-    let max_elements_in_memory = context.cache.max_tensor_elements;
-    let chunk_size = max_elements_in_memory / context.settings.scheduling_num_mem_chunks;
     let dag = circuits_to_dag(&[&**circuit]);
-
+    if circuit.info().max_non_input_size > BigUint::from(context.cache.fallback_total_numel) {
+        return Err(SchedulingError::SingleElementTooBig {
+            numel: circuit.info().max_non_input_size.clone(),
+        });
+    }
     let order_result = {
-        // currently dag simplifications increase achievable memory usage unfortunately
-        // todo make better simplifications
-        if context.settings.scheduling_simplify {
-            // operate on subset dag
-            let (sub_dag, kept_nodes) =
-                simplify_dag_for_scheduling_and_kept_nodes(&dag, chunk_size);
-            if context.settings.verbose >= 2 {
-                println!(
-                    "dag sizes: orig: {} pruned: {}",
-                    dag.node_costs.len(),
-                    sub_dag.node_costs.len()
+        let mut result = Err(SchedulingError::CannotFitInMemory {
+            max_memory: 0,
+            memory_chunks: 0,
+            node_memories: vec![],
+            string: "".to_owned(),
+        });
+        let mut mem = context.cache.max_tensor_elements;
+        while mem <= context.cache.fallback_total_numel && result.is_err() {
+            if context.settings.scheduling_simplify {
+                let chunk_size = mem / context.settings.scheduling_num_mem_chunks;
+                // operate on subset dag
+                let (sub_dag, kept_nodes) = simplify_dag_for_scheduling_and_kept_nodes(
+                    &dag,
+                    chunk_size,
+                    context.settings.scheduling_simplify_lossy,
+                );
+                if context.settings.verbose >= 2 {
+                    println!(
+                        "dag sizes: orig: {} pruned: {}",
+                        dag.node_costs.len(),
+                        sub_dag.node_costs.len()
+                    );
+                }
+                result = schedule_dag_strategy_ints(
+                    &sub_dag,
+                    context.settings.verbose,
+                    mem,
+                    context.settings.scheduling_num_mem_chunks,
+                    context.settings.scheduling_timeout,
+                )
+                .map(|sub_order| subset_order_to_full_order(&sub_order, &kept_nodes, &dag));
+            } else {
+                result = schedule_dag_strategy_ints(
+                    &dag,
+                    context.settings.verbose,
+                    mem,
+                    context.settings.scheduling_num_mem_chunks,
+                    context.settings.scheduling_timeout,
                 );
             }
-            schedule_dag_strategy_ints(
-                &sub_dag,
-                context.settings.verbose,
-                max_elements_in_memory,
-                context.settings.scheduling_num_mem_chunks,
-            )
-            .map(|sub_order| subset_order_to_full_order(&sub_order, &kept_nodes, &dag))
-        } else {
-            schedule_dag_strategy_ints(
-                &dag,
-                context.settings.verbose,
-                max_elements_in_memory,
-                context.settings.scheduling_num_mem_chunks,
-            )
+            mem *= 2;
         }
+        result
     };
     if order_result.is_err() {
         circuit.compiler_print();
