@@ -47,6 +47,7 @@ pub mod lazy;
 pub mod opt_einsum;
 pub mod py_types;
 pub mod rearrange_spec;
+pub mod rrfs;
 pub mod smallvec;
 pub mod tensor_util;
 mod union_find;
@@ -262,6 +263,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(update_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(path_get, m)?)?;
     m.add_function(wrap_pyfunction!(update_path_py, m)?)?;
+    m.add_function(wrap_pyfunction!(rrfs::save_tensor_rrfs, m)?)?;
+    m.add_function(wrap_pyfunction!(rrfs::tensor_from_hash, m)?)?;
 
     // add more py function as desired or whatever...
     Ok(())
