@@ -1,4 +1,4 @@
-use crate::circuit::EinsumAxes;
+use crate::circuit::{Einsum, EinsumAxes, Symbol};
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 /// This file optimizes tensor contraction order, like Python's opt_einsum
 /// It specifies contraction order using rotating id's, like opt_einsum or numpy,
@@ -21,7 +21,9 @@ use pyo3::{
     exceptions::PyValueError,
     types::{PyDict, PyList},
 };
+use uuid::Uuid;
 
+use std::time::Duration;
 use std::{
     cmp::{max, min},
     rc::Rc,
@@ -389,7 +391,7 @@ impl EinsumSpec {
         mem_cap: Option<usize>,
         hash_map_cap: Option<usize>,
     ) -> Option<Vec<Vec<usize>>> {
-        let hash_map_cap = hash_map_cap.unwrap_or(500);
+        let hash_map_cap = hash_map_cap.unwrap_or(70);
         let mem_cap = mem_cap.unwrap_or(usize::MAX);
         if self.input_ints.len() == 1 {
             return Some(vec![vec![0]]);
@@ -409,9 +411,9 @@ impl EinsumSpec {
             let sg_bmp: usize = ints_to_bitmap(sg);
             let mut cost_cap: usize = max(
                 2,
-                bitmap_and_sizes_to_size(sg_bmp & out_bmp, &self.int_sizes),
+                bitmap_and_sizes_to_size(sg_bmp & out_bmp, &self.int_sizes) * 64,
             );
-            let cost_cap_increment: usize = 4;
+            let cost_cap_increment: usize = 8;
             assert!(
                 sg.len() < 64,
                 "store bitmaps as usize, so can only have 64 per subgraph"
@@ -526,7 +528,6 @@ impl EinsumSpec {
             }
 
             subgraph_results.push(dp.last().unwrap()[&sg_bmp].clone());
-            // println!("maxs {}",dp.iter().map(|x|x.len()).max().unwrap());
         }
         let overall_contraction = independent_tree_contraction(
             &subgraph_results.iter().map(|x| (**x).clone()).collect(),
@@ -578,6 +579,27 @@ impl EinsumSpec {
     fn __repr__(&self) -> String {
         format!("{:?}", self)
     }
+
+    pub fn symbol_circuit(&self) -> Einsum {
+        Einsum::try_new(
+            self.input_ints
+                .iter()
+                .map(|ints| {
+                    (
+                        Symbol::nrc(
+                            ints.iter().map(|i| self.int_sizes[*i]).collect(),
+                            Uuid::new_v4(),
+                            None,
+                        ),
+                        ints.iter().map(|i| *i as u8).collect(),
+                    )
+                })
+                .collect(),
+            self.output_ints.iter().map(|i| *i as u8).collect(),
+            None,
+        )
+        .unwrap()
+    }
 }
 
 // non python methods
