@@ -231,7 +231,7 @@ impl RearrangeSpec {
 
     pub fn is_valid(&self) -> bool {
         // check each int appears once in input and output
-        let result = is_unique(&self.ints_in_inp_it().collect::<Vec<_>>())
+        let result = self.ints_in_inp_it().unique().count() == self.ints_in_inp_it().count()
             && is_unique(&self.ints_in_out_it().collect::<Vec<EinInt>>());
         // check each input is in output
         let ints_in_out = self.ints_in_out();
