@@ -598,6 +598,28 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
     TensorIndex(result)
 }
 
+pub fn cuda_to_first_cuda(string: &str) -> String {
+    if string == "cuda" {
+        let number = match std::env::var("CUDA_VISIBLE_DEVICES") {
+            Err(_e) => 0,
+            Ok(s) => s
+                .split(",")
+                .map(|s| s.parse::<usize>())
+                .nth(0)
+                .unwrap_or(Ok(0))
+                .unwrap_or(0),
+        };
+        format!("cuda:{}", number)
+    } else {
+        string.to_owned()
+    }
+}
+
+#[test]
+fn test_cvd() {
+    dbg!(cuda_to_first_cuda("cuda"));
+}
+
 /// use string so you can compare fast when constructing and have full range (weird types like uint or bfloat16)
 /// as opposed to PyObject torch.dtype or enum (could switch to enum)
 #[pyclass]
@@ -620,7 +642,10 @@ impl TorchDeviceDtypeOp {
     #[new]
     #[args(device = "None", dtype = "None")]
     fn new(device: Option<String>, dtype: Option<String>) -> Self {
-        Self { device, dtype }
+        Self {
+            device: device.map(|x| cuda_to_first_cuda(&x)),
+            dtype,
+        }
     }
 }
 
@@ -637,7 +662,10 @@ pub struct TorchDeviceDtype {
 impl TorchDeviceDtype {
     #[new]
     fn new(device: String, dtype: String) -> Self {
-        Self { device, dtype }
+        Self {
+            device: cuda_to_first_cuda(&device),
+            dtype,
+        }
     }
     #[staticmethod]
     #[pyo3(name = "from_tensor")]
