@@ -153,6 +153,17 @@ macro_rules! timed {
         result
     }};
 }
+#[macro_export]
+macro_rules! timed_value {
+    ($x:expr) => {{
+        let timed_macro_now = std::time::Instant::now();
+
+        let result = $x;
+
+        let elapsed = timed_macro_now.elapsed();
+        (result, elapsed)
+    }};
+}
 
 pub trait AsOp<T> {
     fn into_op(self) -> Option<T>;
